import SwiftUI
import FirebaseDatabase

struct AddItemView: View {
    @EnvironmentObject var directory: Directory
    @Binding var adding: Bool
    @State var nameStr  = ""
    @State var phoneStr = ""
    @State var emailStr = ""
    
    var body: some View {
        VStack {
            HStack {
                Text("name:")
                Spacer()
                TextField("name", text: $nameStr)
            }
            HStack {
                Text("phone:")
                Spacer()
                TextField("phone", text:$phoneStr)
            }
            HStack {
                Text("email:")
                Spacer()
                TextField("email", text: $emailStr)
            }
            Button("Add") {
                guard (!nameStr.isEmpty
                        && !phoneStr.isEmpty
                        && !emailStr.isEmpty) else {
                            adding = false
                            return
                        }
                var de = DirEntry(id: nil,
                                  name: nameStr,
                                  phone: phoneStr,
                                  email: emailStr)
                directory.addEntry(entry: &de)
                adding = false
            }
        }.padding()
    }
}

struct DirEntryView: View {
    var entry: DirEntry
    
    var body: some View {
        VStack {
            HStack {
                Text("name:\t").bold()
                Text(entry.name)
                Spacer()
            }
            HStack {
                Text("phone:\t").bold()
                Text(entry.phone)
                Spacer()
            }
            HStack {
                Text("email:\t").bold()
                Text(entry.email)
                Spacer()
            }
        }
    }
}

struct ContentView: View {
    @EnvironmentObject var directory: Directory
    @State var adding = false
    
    var body: some View {
        VStack {
            Text("Directory").font(.title).bold()
            Spacer()
            NavigationView {
                VStack {
                    List {
                        ForEach(0..<directory.entries.keys.count, id: \.self) { i in
                            let key = directory.entries.keys.index(directory.entries.keys.startIndex, offsetBy: i)
                            let entry = directory.entries[key]
                            DirEntryView(entry: entry.value)
                        }.onDelete(perform: deleteEm)
                    }

                    NavigationLink(destination: AddItemView(adding: $adding)
                                    .environmentObject(directory),
                                   isActive: $adding) {
                        Button(action: {adding=true}) {
                            Label("Add Item", systemImage: "plus")
                        }
                    }
                }
            }
        }
    }
    
    private func idByOffset(_ offset: Int) -> String {
        let idx = directory.entries.keys.index( directory.entries.keys.startIndex, offsetBy: offset)
        let (key, _) = directory.entries[idx]
        return key
    }
    
    private func addItem(_ entry: inout DirEntry) {
        withAnimation {
            directory.addEntry(entry: &entry)
        }
    }
    
    private func deleteEm(offsets: IndexSet) {
        withAnimation {
            offsets.map {
                idByOffset($0)
            }.forEach(directory.delEntry)
        }
    }
}

struct ContentView_Previews: PreviewProvider {
    static var previews: some View {
        ContentView().environmentObject(Directory())
    }
}

