//
//  model.swift
//  wednesday
//
//  Created by Pete Keleher on 11/10/21.
//

import Foundation
import FirebaseDatabase

struct DirEntry: Codable, Identifiable {
    var id: String?
    var name: String
    var phone: String
    var email: String
    
    var dict: NSDictionary? {
        if let idStr = id {
            let d = NSDictionary(dictionary: [
               "id": idStr, "name": name, "phone": phone, "email": email ])
            return d
        }
        return nil
    }
    
    static func fromDict(_ d: NSDictionary) -> DirEntry? {
        guard let name = d["name"] as? String else { return nil }
        guard let phone = d["phone"] as? String else { return nil }
        guard let email = d["email"] as? String else { return nil }
        return DirEntry(id: d["id"] as? String, name: name, phone: phone, email: email)
    }
}
 
class Directory: ObservableObject {
    @Published var entries: [String:DirEntry] = [:]
    
    init() {
        let rootRef = Database.database().reference()
        rootRef.getData { err, snapshot in
            DispatchQueue.main.async {
                for child in snapshot.children {
                    if let item = child as? DataSnapshot {
                        if let val = item.value as? NSDictionary,
                           let de = DirEntry.fromDict(val),
                           let id = de.id { self.entries[id] = de }
                    }
                }
            }
        }
        rootRef.observe(.childAdded) { snapshot in
            if let v = snapshot.value as? NSDictionary,
               let de = DirEntry.fromDict(v),
               let id = de.id { self.entries[id] = de }
        }
        rootRef.observe(.childRemoved) { snapshot in
            self.entries.removeValue(forKey: snapshot.key)
        }
        rootRef.observe(.childChanged) { snapshot in
            if let v = snapshot.value as? NSDictionary,
               let de = DirEntry.fromDict(v),
               let id = de.id { self.entries[id] = de }
        }
    }
    func addEntry(entry: inout DirEntry) {
        let rootRef = Database.database().reference()
        let childRef = rootRef.childByAutoId()
        entry.id = childRef.key
        if let val = entry.dict {
            childRef.setValue(val)
        }
    }
    
    func delEntry(id: String) {
        entries.removeValue(forKey: id)
        let rootRef = Database.database().reference()
        rootRef.child(id).removeValue()
    }
}


